/*
 *  LinkLabel.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using GLib;
using Gtk;
using Gdk;
using Mono.Unix;


namespace Incollector.GUI.Widgets {
	
	public class LinkLabel : Gtk.HBox {
		
		protected Gtk.Label label;
		protected string    Url;
		
		protected TargetEntry [] TargetTable = new TargetEntry [] {
			new TargetEntry ("STRING", 0, 0)
		};
		
		
		public float Xalign {
			get { return label.Xalign;  }
			set { label.Xalign = value; }
		}
		
		public float Yalign {
			get { return label.Yalign;  }
			set { label.Yalign = value; }
		}
		
		
		public LinkLabel (string Text, string Url)
		{
			CreateLinkLabel (Text, Url);
		}
		
		
		public LinkLabel (string Url)
		{
			CreateLinkLabel (Url, Url);
		}
		
		
		public void CreateLinkLabel (string Text, string Url)
		{
			this.Url = Url;
			
			Gtk.EventBox evbox = new Gtk.EventBox ();
			evbox.VisibleWindow = false;
			evbox.ButtonPressEvent += new ButtonPressEventHandler (on_Pressed);
			evbox.ButtonReleaseEvent += new ButtonReleaseEventHandler (on_Clicked);
			
			evbox.EnterNotifyEvent += new EnterNotifyEventHandler (on_Enter);
			evbox.LeaveNotifyEvent += new LeaveNotifyEventHandler (on_Leave);
			evbox.DragDataGet += new DragDataGetHandler (on_drag_data_get);
			
			Gtk.Drag.SourceSet (evbox, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
				    TargetTable, DragAction.Copy | DragAction.Move);
			
			label = new Label (Url);
			label.ModifyFg (Gtk.StateType.Normal, new Gdk.Color (66, 98, 235));
			label.UseUnderline = false;
			label.Text   = Text;
			label.Xalign = 0;
			label.Yalign = (float) 0.50;
			evbox.Add (label);
			
			this.Add (evbox);
			
			
			this.ShowAll ();
		}
		
		
		protected void on_drag_data_get (object o, DragDataGetArgs args)
		{
			byte [] data = System.Text.Encoding.UTF8.GetBytes (this.Url);
			Gdk.Atom [] targets = args.Context.Targets;
			
			args.SelectionData.Set (targets[0], 8, data, data.Length);
		}
		
		
		protected void on_Enter (object o, EnterNotifyEventArgs args)
		{
			label.ModifyFg (Gtk.StateType.Normal, new Gdk.Color (96, 128, 255));
			label.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Hand2);
		}
		
		
		protected void on_Leave (object o, LeaveNotifyEventArgs args)
		{
			label.ModifyFg (Gtk.StateType.Normal, new Gdk.Color (66, 98, 235));
			label.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.TopLeftArrow);
		}
		
		
		protected void on_Pressed (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 3) {
				Menu menu = new Menu ();
				
				Gtk.ImageMenuItem OpenLink = new Gtk.ImageMenuItem (Catalog.GetString ("Open link"));
				OpenLink.Image = new Gtk.Image (Stock.JumpTo, Gtk.IconSize.Menu);
				OpenLink.Activated += new EventHandler (on_OpenLink_activate);
				menu.Add (OpenLink);
				
				Gtk.ImageMenuItem CopyLink = new Gtk.ImageMenuItem (Catalog.GetString ("Copy link address"));
				CopyLink.Image = new Gtk.Image (Stock.Copy, Gtk.IconSize.Menu);
				CopyLink.Activated += new EventHandler (on_CopyLink_activate);
				menu.Add (CopyLink);
				
				menu.ShowAll ();
				menu.Popup ();
			}
		}
		
		
		protected void on_Clicked (object o, ButtonReleaseEventArgs args)
		{
			if ((args.Event.X != 0) && ((args.Event.Y != 0)) && (args.Event.Button == 1))
				Tools.RunWebBrowser (Url);
		}
		
		
		protected void on_OpenLink_activate (object o, EventArgs args)
		{
			Tools.RunWebBrowser (label.Text);
		}
		
		
		protected void on_CopyLink_activate (object o, EventArgs args)
		{
			Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", true)).Text = Url;
		}
	}
}
